local _t, _t2, _, T = {}, {}, ...

local function GetPetTypeIcon(petType)
	return "|TInterface\\PetBattles\\PetIcon-"..PET_TYPE_SUFFIX[petType] .. ":0:0:0:0:128:256:102:63:129:168|t"
end
local function SetZoomedIcon(self, tex)
	self:SetTexture(tex)
	self:SetTexCoord(6/64,60/64, 6/64,60/64)
end
local GetPetAbilities do
	local _t, _t2 = {}, {}
	function GetPetAbilities(species)
		wipe(_t)
		return C_PetJournal.GetPetAbilityList(species, _t, _t2)
	end
end
local GetFamilyColor do
	local c = {}
	for r,g,b in ("05a9fd 4baa14 dad15c a46d75 7c5a43 b479ff FCA800 f32f24 06bdc9 9b978b"):gmatch("(%x%x)(%x%x)(%x%x)") do
		c[#c+1], c[#c+2], c[#c+3] = tonumber(r,16)/255, tonumber(g,16)/255, tonumber(b,16)/255
	end
	function GetFamilyColor(id)
		return c[3*id-2], c[3*id-1], c[3*id]
	end
end

do -- LittleBattlerTip
	local function pickAnchor(self)
		local l, b, w, h = self:GetRect()
		local pl, pb, pw, ph = self:GetParent():GetRect()
	
		if (l - pl) < 0.2*pw then
			return "LEFT", self, "RIGHT", 5, 0
		elseif (l - pl) > 0.8*pw then
			return "RIGHT", self, "LEFT", -5, 0
		elseif (b - pb) < 0.5*ph then
			return "BOTTOM", self, "TOP", 0, -5
		else
			return "TOP", self, "BOTTOM", 0, 5
		end
	end
	function LittleBattlerTip:SetTamer(poi, name, title, tamer, eligible, completed)
		local p1, p2, p3, lvl, ql = T.GetTamerTeam(tamer)
		self:ClearAllPoints()
		self:SetPoint(pickAnchor(poi))
		self:SetOwner(poi, "ANCHOR_PRESERVE")
		self:AddLine(name, 1,1,1)
		if not p1 and (title or "") ~= "" and lvl and ql then
			title = "|c" .. select(4, GetItemQualityColor(ql-1)) .. "[" .. lvl .. "]|r " .. title
		end
		if (title or "") ~= "" then self:AddLine(title, nil, nil, nil, 1) end
		if p1 then self:AddLine(" ") end
		for i=1,p3 and 3 or p2 and 2 or p1 and 1 or 0 do
			local sid = i == 3 and p3 or i == 2 and p2 or p1
			local r, name, icon, ptype = "", C_PetJournal.GetPetInfoBySpeciesID(sid)
			local abilities = GetPetAbilities(sid)
			for i=1,#abilities do
				local i, _, _, _, _, _, pt, nsw = C_PetBattles.GetAbilityInfoByID(abilities[i])
				if not nsw then
					r = r .. " " .. GetPetTypeIcon(pt)
				end
			end
			self:AddDoubleLine(("%s |T%s:0:0:0:0:64:64:6:58:6:58|t |c%s[%d]|r %s"):format(
				GetPetTypeIcon(ptype), icon, select(4,GetItemQualityColor(ql-1)), lvl, name ~= "" and name or T.GetTamerPetName(sid)), r)
		end
		if completed then
			self:AddLine(" ")
			self:AddLine(GREEN_FONT_COLOR_CODE .. "|TInterface\\Scenarios\\ScenarioIcon-Check:0|t Daily quest completed")
		elseif not eligible then
			self:AddLine(" ")
			self:AddLine(GRAY_FONT_COLOR_CODE .. "|TInterface\\PetBattles\\PetBattle-LockIcon:0|t Daily not unlocked")
		elseif eligible == "faction" then
			self:AddLine(" ")
			self:AddLine(GRAY_FONT_COLOR_CODE .. "|TInterface\\PetBattles\\PetBattle-LockIcon:0|t Faction locked")
		end
		self:SetScale(UIParent:GetScale())
		self:Show()
	end
end

local function poiEnter(self)
	local tamer, eligible, completed, name, title
	if self.tamerId then
		tamer, eligible, completed = self.tamerId, T.GetTamerStatus(self.tamerId)
		name, title = T.GetTamerName(tamer)
	else
		name, title, tamer, eligible, completed = self.name, self.description, T.IsTamerLandmark(tonumber(self:GetName():match("%d+$")))
	end
	if tamer then
		WorldMapFrameAreaLabel:SetText("")
		WorldMapFrameAreaDescription:SetText("")
		LittleBattlerTip:SetTamer(self, name, title, tamer, eligible, completed)
	end
end
local function poiLeave(self)
	if LittleBattlerTip:IsOwned(self) then
		LittleBattlerTip:Hide()
	end
end
local function poiReset(self)
	self.Texture:SetAlpha(1)
	self.Texture:SetDesaturated(nil)
	self:SetFrameLevel(WorldMapButton:GetFrameLevel()+1)
end

local function createPOI(t, k)
	local button = CreateFrame("Button", nil, WorldMapButton)
	button:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	button:SetScript("OnEnter", poiEnter)
	button:SetScript("OnLeave", poiLeave)

	button.Texture = button:CreateTexture()

	WorldMap_ResetPOI(button, isObjectIcon)
	t[k] = button
	return button
end
local ownPOIs = setmetatable({}, {__index=createPOI})

local function worldHide()
	local o = LittleBattlerTip:GetOwner()
	if o and o:GetParent() == WorldMapButton then
		LittleBattlerTip:Hide()
	end
end
local function updateLandmark(poi, tamer, eligible, completed)
	if eligible ~= true then
		poi.Texture:SetAlpha(eligible == "faction" and 0.4 or 0.6)
	end
	if completed then
		poi.Texture:SetDesaturated(1)
	end
	poi:SetFrameLevel(WorldMapButton:GetFrameLevel()+2)
end
local function addExtraMarks(i, tamer, x, y, ...)
	if tamer then
		local poi, _, _, w, h = ownPOIs[i], WorldMapButton:GetRect()
		poi:Show()
		poi:SetPoint("CENTER", WorldMapButton, "TOPLEFT", x/1e5*w, -y/1e5*h)
		poi.Texture:SetTexCoord(GetPOITextureCoords(196))
		poi.tamerId = tamer
		updateLandmark(poi, tamer, T.GetTamerStatus(tamer))
		return addExtraMarks(i+1, ...)
	else
		for i=i,#ownPOIs do
			ownPOIs[i]:Hide()
		end
	end
end
local function worldUpdate()
	for i=1,GetNumMapLandmarks() do
		local tamer, eligible, completed = T.IsTamerLandmark(i)
		if tamer then
			updateLandmark(_G["WorldMapFramePOI" .. i], tamer, eligible, completed)
		end
	end
	addExtraMarks(1, T.GetExtraLandmarks(GetCVarBool("showTamers") and GetCurrentMapAreaID()))
end

local function gossipUpdate()
	local id = T.IsTamerNPC(tonumber((UnitGUID("npc") or "0"):sub(6, 10), 16))
	if id then
		local last = _G["GossipTitleButton" .. (GossipFrame.buttonIndex-1)]
		if last and last.type == "Gossip" then
			local tex = _G[last:GetName() .. "GossipIcon"]
			tex:SetTexture([[Interface\PetBattles\PetBattle-StatIcons]])
			tex:SetTexCoord(0, 0.5, 0, 0.5)
		end
		local button = _G["GossipTitleButton" .. GossipFrame.buttonIndex]
		local tex = _G[button:GetName() .. "GossipIcon"]
		tex:SetTexture([[Interface\GossipFrame\GossipGossipIcon]])
		button:SetID(GossipFrame.buttonIndex)
		button.type, button.lbTamerId = "LittleBattler", id
		button:SetText("Tell me more about your pets...")
		button:Show()
		GossipResize(button)
		GossipFrame.buttonIndex = GossipFrame.buttonIndex + 1
	end
end
local function gossipSelect(id)
	local btn = _G["GossipTitleButton" .. id]
	if btn and btn.type == "LittleBattler" then
		btn:Hide()
		
		local p1, p2, p3, lvl, ql = T.GetTamerTeam(btn.lbTamerId)
		if not p1 then
			GossipGreetingText:SetText("This is a little embarassing, but I don't know anything about my battle pet team. Why don't we battle it out instead?")
			return
		end
		
		local numPets = p3 and 3 or p2 and 2 or 1
		LittleBattlerTeamInfo:SetHeight(76*numPets - 16)
		GossipGreetingText:SetText("QWP")
		local lineHeight, teamHeight = GossipGreetingText:GetStringHeight(), 25+LittleBattlerTeamInfo:GetHeight()
		GossipGreetingText:SetText("My team is going to bury your team!")
		local greetHeight = GossipGreetingText:GetStringHeight()
		LittleBattlerTeamInfo:SetPoint("TOPLEFT", GossipGreetingText, "TOPLEFT", 0, -greetHeight-25)
		GossipGreetingText:SetText("My team is going to bury your team!" .. ("\n"):rep(math.max(0,math.ceil(teamHeight/lineHeight))))	
		LittleBattlerTeamInfo:Show()
		
		local icof = "|TInterface\\PetBattles\\PetBattle-StatIcons:0:0:0:0:32:32:%d:%d:%d:%d|t %s "
		for i=1,numPets do
			local pid, pet = i == 3 and p3 or i == 2 and p2 or p1, LittleBattlerTeamInfo.pets[i]
			local name, ico, ptype = C_PetJournal.GetPetInfoBySpeciesID(pid)
			local hp, ap, sp, rlvl = T.GetTamerPetStats(pid)
			pet:Show()
			pet.name:SetText((name or "") ~= "" and name or T.GetTamerPetName(pid))
			pet.level:SetText("|c" .. select(4, GetItemQualityColor((ql or 2)-1)) .. (rlvl or lvl or "?"))
			pet.iconButton.id = PET_BATTLE_PET_TYPE_PASSIVES[ptype]
			SetZoomedIcon(pet.icon, ico)
			pet.iconBorder:SetVertexColor(GetFamilyColor(ptype))
			pet.levelBG:SetVertexColor(GetFamilyColor(ptype))
			pet.stats:SetFormattedText("%s%s%s", hp and icof:format(16,32,16,32, hp) or "", sp and icof:format(0,16,16,32, sp) or "", ap and icof:format(0,16,0,16, ap) or "")
			pet.id, pet.ap, pet.sp, pet.hp = pid, ap, sp, hp
			local aid = GetPetAbilities(pid)
			for j=1,#aid do
				local ab, name, icon, atype = pet.abilities[j], C_PetJournal.GetPetAbilityInfo(aid[j])
				SetZoomedIcon(ab.icon, icon)
				ab.iconBorder:SetVertexColor(GetFamilyColor(atype))
				ab.id = aid[j]
			end
		end
		for i=numPets+1,3 do
			LittleBattlerTeamInfo.pets[i]:Hide()
		end
	end
end
local function gossipConfirm(_, event, gid)
	if event == "GOSSIP_CONFIRM" and T.IsTamerNPC(tonumber((UnitGUID("npc") or "0"):sub(6, 10), 16)) then
		T.RaiseEvent("PET_BATTLE_SOON")
		SelectGossipOption(gid, "", true)
		StaticPopup_Hide(event)
	end
end
local function gossipCleanup()
	local i, b = 2, GossipTitleButton1
	while b do
		if b.lbTamerId then
			b.lbTamerId = nil
			_G[b:GetName() .. "GossipIcon"]:SetTexCoord(0,1, 0,1)
		end
		i, b = i + 1, _G["GossipTitleButton" .. i]
	end
end

WorldMapPlayerUpper:EnableMouse(nil)
WorldMapFrame:HookScript("OnHide", worldHide)
hooksecurefunc("WorldMapFrame_Update", worldUpdate)
hooksecurefunc("WorldMapFrame_UpdateMap", worldUpdate)
hooksecurefunc("WorldMapPOI_OnEnter", poiEnter)
hooksecurefunc("WorldMapPOI_OnLeave", poiLeave)
hooksecurefunc("WorldMap_ResetPOI", poiReset)
hooksecurefunc("GossipFrameUpdate", gossipUpdate)
hooksecurefunc("SelectGossipOption", gossipSelect)
UIParent:HookScript("OnEvent", gossipConfirm)
GossipFrameGreetingPanel:HookScript("OnHide", gossipCleanup)
local i, f = 2, WorldMapFramePOI1 while f do
	f:HookScript("OnEnter", poiEnter)
	f:HookScript("OnLeave", poiLeave)
	i, f = i + 1, _G["WorldMapFramePOI" .. i]
end